from crypto_connector.binance import Binance  # noqa: F401
from crypto_connector.bybit import Bybit  # noqa: F401
from crypto_connector.htx import HTX  # noqa: F401

exchanges = ["Binance", "Bybit", "HTX"]
